'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os
import pygame

dirname = os.path.abspath(os.path.dirname(__file__))

def load(filename):
    f = None
    try:
        f = open(os.path.join(dirname, '..', 'data', filename), 'rb')
    except:
        print "Error getting file handle: %s" % (os.path.join(dirname, '..', 'data', filename), 'rb')
    return f

def path(filename):
    return os.path.join(dirname, '..', 'data', filename)

def img_load(filename, frame=1, count=0, tile_width=16, tile_height=16):
    """Loads an image from file, splits it into pygame surfaces separated by "frame" pixels according to tile_width/tile_height.
        Loads left to right, top to bottom, up to count surfaces, or if count is 0, as many as possible.

        Uses upper left-most pixel as colorkey for transparency
    """

    tileset = []

    try:
        image = pygame.image.load(path(filename))
    except:
        print "Error loading tileset from file: %s" % (path(filename))
        return

    image.convert()

    # Get color to pixel at 0,0
    colorkey = image.get_at((0,0))

    width = image.get_width()
    height = image.get_height()

    cols = width / tile_width
    rows = height / tile_height

    # Figure out what our max possible tiles are
    maxcount = rows * cols

    if (count == 0) or (count > maxcount):
        count = maxcount

    # Now get the tiles sorted out
    x = 0
    y = 0
    loaded = 0
    for i in range(rows):
        # The last + frame compensates for starting frame
        y = (i * tile_height) + (i * frame) + frame

        for j in range(cols):
            # The last + frame compensates for starting frame
            x = (j * tile_width) + (j * frame) + frame

            # Create a temp surface and copy image to it
            tmpsurf = pygame.Surface((tile_width, tile_height))
            tmpsurf.blit(image, (0,0), (x, y, tile_width, tile_height))
            tmpsurf.convert()               # Just in case
            tmpsurf.set_colorkey(colorkey)  # Set the color that will be transparent

            tileset.append(tmpsurf)

    # Debug
    #print image, width, height, rows, cols, count

    return tileset
