"""
    Written by Aaron Hachez, 2007

    I'm going to put everything in plain files, no special loading routines.

    Get scrolling up and working.
    Get movement up and running.
    Make a simple level.
"""

debug = None

import sys
import os
import pygame
from pygame.locals import *

## Skellington
import data

## My imports
from config import config

config.width = 640
config.height = 480

from input import input
from map import map
from constants import *

def init():
    #### Init

    (init_success, init_fail) = pygame.init()


    screen = None
    if pygame.display.get_init():
        screen = pygame.display.set_mode((config.get_width(), config.get_height()), FULLSCREEN)
        pygame.display.set_caption("Onomatopoeia Editor -- Pyweek 4 Entry by Aaron Hachez")
        # TODO: Set icon
    else:
        print "The display system failed to initialize, exiting..."
        sys.exit(1)

    if not pygame.font.get_init():
        print "Error, font system didn't initialize properly, the game may not play properly."
    if not pygame.mixer.get_init():
        print "Error, sound system didn't initialize properly, the game will probably play properly, but you will not have sound."

    return screen

def shutdown():
    pygame.quit()

def draw_tiles(screen, tileset):
    """Draw the available tiles to the screen"""
    x = 0
    y = 0
    index = 0
    count = len(tileset)
    for j in range(screen.get_height() / 16):
        y = (j * 16) + 16
        for i in range(screen.get_width() / 16):
            x = (i * 16)
            screen.blit(tileset[index], (x,y))
            index = index + 1
            if index >= count:
                break
        if index >= count:
            break

def rect_gotclick((mx, my), (bx,by, bw,bh)):
    """Determine if a rect got clicked"""

    if mx < bx:
        pass
    elif my < by:
        pass
    elif mx > by + bw:
        pass
    elif my > by + bh:
        pass
    else:
        return True

    return None

def select_tile(screen, tileset, (mx,my)):
    """Draw the available tiles to the screen"""
    x = 0
    y = 0
    index = 0
    count = len(tileset)
    for j in range(screen.get_height() / 16):
        y = (j * 16) + 16
        for i in range(screen.get_width() / 16):
            x = (i * 16)

            if mx < x:
                pass
            elif my < y:
                pass
            elif my > y + 16:
                pass
            elif mx > x + 16:
                pass
            else:
                return index

            index = index + 1
            if index >= count:
                break
        if index >= count:
            break

    return -1

def main():

    screen = init()
    font = pygame.font.Font(None, 16)
    clock = pygame.time.Clock()

    tile_list = data.img_load("level01.png")
    map_name = "three.map"
    #map.fill_mapdata(map_name, "level01.png", 0, 20, 50)
    map.load(map_name)

    #### Main program
    
    mapsurf = pygame.Surface((320, 240))
    mid_x = (screen.get_width() - 320) / 2
    mid_y = (screen.get_height() - 240) / 2

    mx = 0
    my = 0
    lbutton = None
    rbutton = None
    mbutton = None

    selected_tile = 1

    # Available meta:
    # pass, jump_through, monster_stop
    #    0,            1,            2
    selected_meta = META_PASS

    tmp_button = None

    pass_button = font.render(" pass_through ", 1, (255,255,255))
    tmp_button = pygame.Surface((pass_button.get_width(), pass_button.get_height()))
    tmp_button.fill((0,0,0))
    tmp_button.blit(pass_button, (0,0))
    pass_button = tmp_button
    
    jump_button = font.render(" jump_through ", 1, (255,255,255))
    tmp_button = pygame.Surface((jump_button.get_width(), jump_button.get_height()))
    tmp_button.fill((0,0,0))
    tmp_button.blit(jump_button, (0,0))
    jump_button = tmp_button
    
    monster_button = font.render(" monster_stop ", 1, (255,255,255))
    tmp_button = pygame.Surface((monster_button.get_width(), monster_button.get_height()))
    tmp_button.fill((0,0,0))
    tmp_button.blit(monster_button, (0,0))
    monster_button = tmp_button

    none_button = font.render(" none ", 1, (255,255,255))
    tmp_button = pygame.Surface((none_button.get_width(), none_button.get_height()))
    tmp_button.fill((0,0,0))
    tmp_button.blit(none_button, (0,0))
    none_button = tmp_button
 

    active = 1
    quit = None
    while(not quit):
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                if event.key == K_F1:
                    map.save(map_name)
                    print map_name, "saved"

                input.proc_down(event)

            elif event.type == KEYUP:
                input.proc_up(event)

            elif event.type == MOUSEBUTTONDOWN:
                (lbutton, mbutton, rbutton) = pygame.mouse.get_pressed()

            elif event.type == MOUSEBUTTONUP:
                (lbutton, mbutton, rbutton) = pygame.mouse.get_pressed()

            elif event.type == ACTIVEEVENT:
                active = event.dict['gain']

            elif event.type == QUIT:
                quit = 1

        (mx, my) = pygame.mouse.get_pos()

        if lbutton or mbutton or rbutton :

            b_x = 0
            b_y = 70
            if rect_gotclick((mx, my), (b_x, b_y, pass_button.get_width(), pass_button.get_height())):
                selected_meta = META_PASS
            b_y = b_y + pass_button.get_height() + 2

            if rect_gotclick((mx, my), (b_x, b_y, jump_button.get_width(), jump_button.get_height())):
                selected_meta = META_JUMP
            b_y = b_y + jump_button.get_height() + 2

            if rect_gotclick((mx, my), (b_x, b_y, monster_button.get_width(), monster_button.get_height())):
                selected_meta = META_MONSTER
            b_y = b_y + monster_button.get_height() + 2

            if rect_gotclick((mx, my), (b_x, b_y, none_button.get_width(), none_button.get_height())):
                selected_meta = META_NONE

            tmptile = select_tile(screen, tile_list, (mx, my))
            if tmptile != -1:
                selected_tile = tmptile

            # Check for map click
            if mx < mid_x:
                pass
            elif my < mid_y:
                pass
            elif mx > mid_x + 320:
                pass
            elif my > mid_y + 240:
                pass
            else:
                # It's a map click
                tile = map.point_collision((mx - mid_x, my - mid_y))

                if lbutton:
                    map.set_tile(tile, selected_tile, selected_meta)
                elif rbutton:
                    map.set_tile(tile, -1, None)

            print selected_meta, selected_tile, mx, my

        if input.get_left():
            map.set_viewport((map.get_x() - 1, map.get_y()))
        elif input.get_right():
            map.set_viewport((map.get_x() + 1, map.get_y()))
        elif input.get_up():
            map.set_viewport((map.get_x(), map.get_y() - 1))
        elif input.get_down():
            map.set_viewport((map.get_x(), map.get_y() + 1))

        # TODO: For debug only, remove for production
        screen.fill((0,90,0))

        b_x = 0
        b_y = 70
        screen.blit(pass_button, (b_x, b_y))
        b_y = b_y + pass_button.get_height() + 2
        screen.blit(jump_button, (b_x, b_y))
        b_y = b_y + jump_button.get_height() + 2
        screen.blit(monster_button, (b_x, b_y))
        b_y = b_y + monster_button.get_height() + 2
        screen.blit(none_button, (b_x, b_y))

        draw_tiles(screen, tile_list)

        screen.fill((50,50,50), (mid_x - 10, mid_y - 10, 340, 260))
        mapsurf.fill((0,0,0))
        map.debug_draw(mapsurf)
        screen.blit(mapsurf, (mid_x, mid_y))

        clock.tick()

        if config.get_debug():
            string = "fps: %f" % (clock.get_fps())
            fpsdisplay = font.render(string, 1, (200,200,200))
            screen.blit(fpsdisplay, (1, config.get_height() - fpsdisplay.get_height()))

        pygame.display.flip()

    shutdown()

main()
