"""
    Written by Aaron Hachez, 2007

    I'm going to put everything in plain files, no special loading routines.

    Get scrolling up and working.
    Get movement up and running.
    Make a simple level.
"""

import sys
import os
import pygame
from pygame.locals import *

## Skellington
import data

## My imports
from config import config
from constants import *
from input import input
from map import map
from object import object_mgr
from object import Object
from player import Player

import pdb

def init():
    #### Init

    (init_success, init_fail) = pygame.init()

    screen = None
    if pygame.display.get_init():
        if config.get_scale2x():
            screen = pygame.display.set_mode((config.get_width() * 2, config.get_height() * 2))
        else:
            screen = pygame.display.set_mode((config.get_width(), config.get_height()))

        pygame.display.set_caption("Onomatopoeia -- Pyweek 4 Entry by Aaron Hachez")
        # TODO: Set icon
    else:
        print "The display system failed to initialize, exiting..."
        sys.exit(1)

    if not pygame.font.get_init():
        print "Error, font system didn't initialize properly, the game may not play properly."
    if not pygame.mixer.get_init():
        print "Error, sound system didn't initialize properly, the game will probably play properly, but you will not have sound."

    # Always start with map one
    map.load("three.map")

    return screen

def shutdown():
    object_mgr.shutdown()
    pygame.quit()

def main():

    screen = init()
    font = pygame.font.Font(None, 24)
    clock = pygame.time.Clock()

    #### testing
    #               name           , x,  y, tw, th, anim
    player = Player(TYPE_PLAYER)
    player.set((2,23))
    player.add_animation([2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
    #player.add_animation([1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1])
    #player.add_animation([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
    player.push_state(0, None, None)
    #player.push_state(2, None, None)
    #player.push_state(1, None, 40)

#    obj2 = Object("Jill, the girl?", 23, 30, 16, 16, 0)
#    obj2.load_tiles("tiletest.png")
#    obj2.add_animation([1, 0, 0, 0, 0])
#    obj2.push_state(0, None, None)

    object_mgr.add(player)
#    object_mgr.add(obj2)
    
    #### Main program

    # If we're supposed to scale
    if config.get_scale2x():
        backbuf = pygame.Surface((config.get_width(), config.get_height()))
        screenbuf = pygame.Surface((config.get_width() * 2, config.get_height() * 2))
    
    active = 1
    quit = None
    while(not quit):
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                input.proc_down(event)

            elif event.type == KEYUP:
                input.proc_up(event)

            elif event.type == MOUSEBUTTONDOWN:
                map.set_start(pygame.mouse.get_pos())

            elif event.type == ACTIVEEVENT:
                active = event.dict['gain']

            elif event.type == QUIT:
                quit = 1

        # TODO: For debug only, remove for production
        if config.get_scale2x():
            backbuf.fill((200,200,200))
        else:
            screen.fill((200,200,200))

        # Determine where to scroll the map to in order to center the player
        sx = player.get_x() - config.get_center_x() + (player.get_width() / 2)
        sy = player.get_y() - config.get_center_y() + (player.get_height() / 2)
        (sx, sy) = map.translate_screen_to_map((sx, sy))
        (px, py) = map.set_viewport((sx, sy))
        player.move((px, py))   # Adjust for the map position move

        # TODO: improve scale code
        if config.get_scale2x():
            map.debug_draw(backbuf)
            object_mgr.update(1)
            object_mgr.draw(backbuf)

            screenbuf = pygame.transform.scale(backbuf, (config.get_width(), config.get_height()))
            screen.blit(screenbuf, (0,0))
        else:
            #map.debug_draw(screen)
            map.draw(screen)
            object_mgr.update(1)
            object_mgr.draw(screen)
 
        clock.tick(60)

        if config.get_debug():
            string = "fps: %.2f" % (clock.get_fps())
            fpsdisplay = font.render(string, 1, (255,55,55))
            screen.blit(fpsdisplay, (1, config.get_height() - fpsdisplay.get_height()))

        pygame.display.flip()

    shutdown()
