"""
    Main map functions
"""

import cPickle
import pygame

import data
from config import config
from constants import *

class Map:
    def __init__(self):
        self.reset()
        #self.load("foo")

    def reset(self):
#        self.name = None
#
#        # Setup a default data... Just cause
#        # Given screen size 320x240, we have 20x15 tiles
#        # 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
#        self.data = [
#            0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,0,0,0,1,1,1,1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,-1,-1,1,0,
#            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,-1,-1,1,0
#        ]

        self.name = None
        self.tilename = None
        self.data = []
        self.meta = []
        self.tileset = []

        # Defaults, non-changing
        self.tile_width = 16            ## The tile's number of pixels wide
        self.tile_height = 16           ## The tile's number of pixels high
        self.screen_width_tiles = 21    ## The width of the screen in tiles
        self.screen_height_tiles = 16   ## The height of the screen in tiles

        # Resets
        self.viewport_x = float(0.0)    ## The x location to draw at
        self.viewport_y = float(0.0)    ## The y location to draw at
        self.map_width_tiles = 24       ## The width of the map in tiles
        self.map_height_tiles = 24      ## The height of the map in tiles
        self.offset = 3                 ## The number of tiles difference in map width vs screen width
        self.start = 0                  ## The tile to start drawing with
        self.x_offset = float(0.0)      ## The pixel offset to draw with
        self.y_offset = float(0.0)      ## The pixel offset to use in drawing

    def fill_mapdata(self, name, tilename, start, mw, mh):
        """Fills in data and meta and sets initial junk."""

        self.name = name
        self.tilename = tilename
        self.start = start
        self.map_width_tiles = mw
        self.map_height_tiles = mh

        self.data = []
        self.meta = []
        for i in range(mw * mh):
            self.data.append(-1)
            self.meta.append(META_NONE)

        self.offset = self.map_width_tiles - self.screen_width_tiles

        ## Load in the tiles
        self.tileset = data.img_load(self.tilename)

    def load(self, filename):
        """Load a map file"""

        filepath = data.path(filename)
        f = open(filepath, 'rb')

        self.name = cPickle.load(f)
        self.tilename = cPickle.load(f)
        self.start = cPickle.load(f)
        self.map_width_tiles = cPickle.load(f)
        self.map_height_tiles = cPickle.load(f)
        self.data = cPickle.load(f)
        self.meta = cPickle.load(f)

        f.close()

        ## The number of tiles difference in map width vs screen width
        self.offset = self.map_width_tiles - self.screen_width_tiles

        ## Load in the tiles
        self.tileset = data.img_load(self.tilename)

        # The width of the map in pixels
        self.map_width = self.map_width_tiles * self.tile_width
        self.map_height = self.map_height_tiles * self.tile_height

    def save(self, filename):
        """Save the current map out to filename. Very raw, just for editor."""

        filepath = data.path(filename)

        f = open(filepath, 'wb')

        cPickle.dump(filename, f)
        cPickle.dump(self.tilename, f)
        cPickle.dump(self.start, f)
        cPickle.dump(self.map_width_tiles, f)
        cPickle.dump(self.map_height_tiles, f)
        cPickle.dump(self.data, f)
        cPickle.dump(self.meta, f)

        f.close()

    def debug_draw(self, screen):
        """Draw self to screen"""
        
        x = 0
        y = 0

        index = self.start
        for j in range(self.screen_height_tiles):
            y = (j * self.tile_height) - self.y_offset
            for i in range(self.screen_width_tiles):
                x = (i * self.tile_width) - self.x_offset
                tile = self.data[index]
                meta = self.meta[index]

                # Only draw the tiles we need
                if (-1 < tile):
                    screen.blit(self.tileset[tile], (x,y))

                if META_PASS == meta:
                    pygame.draw.rect(screen, (255, 0, 0), (x, y, self.tile_width, self.tile_height), 1)
                elif META_JUMP == meta:
                    pygame.draw.rect(screen, (0, 255, 0), (x, y, self.tile_width, self.tile_height), 1)
                elif META_MONSTER == meta:
                    pygame.draw.rect(screen, (0, 0, 255), (x, y, self.tile_width, self.tile_height), 1)

                index = index + 1

            index = index + self.offset


    def draw(self, screen):
        """Draw self to screen"""
        
        x = 0
        y = 0

        index = self.start
        for j in range(self.screen_height_tiles):
            y = (j * self.tile_height) - self.y_offset
            for i in range(self.screen_width_tiles):
                x = (i * self.tile_width) - self.x_offset
                tile = self.data[index]

                # Only draw the tiles we need
                if (-1 < tile):
                    screen.blit(self.tileset[tile], (x,y))

                index = index + 1

            index = index + self.offset

    def object_collision(self, object):
        """Just calls collsion(self, (x, y, width, height))"""

        return self.collision(object.type, object.x, object.y, object.width, object.height)

    def collision(self, o_type, o_x, o_y, o_width, o_height):
        """Returns two numbers indicating the x and y overlap,
            two values indicating whether the collision happened on the left or right
            and the rectangle of the tile that this object collided with

            (x, y, left, top, rect)

            left or top will be True or None

            All will be None on fail
        """

        x_overlap = 0
        y_overlap = 0

        x = 0
        y = 0
        index = self.start
        for j in range(self.screen_height_tiles):
            y = (j * self.tile_height) - self.y_offset
            for i in range(self.screen_width_tiles):
                x = (i * self.tile_width) - self.x_offset
                tile = self.data[index]
                meta = self.meta[index]

                # If we have a non-blank tile
                if (META_NONE != tile) and (META_PASS != meta) and not ((META_MONSTER == meta) and (TYPE_PLAYER == o_type)):
                    top = None
                    left = None
                
                    obj_rect  = pygame.Rect(o_x, o_y, o_width, o_height)
                    tile_rect = pygame.Rect(x, y, self.tile_width, self.tile_height)
                    
                    if obj_rect.colliderect(tile_rect):
                        overlap = obj_rect.clip(tile_rect)

                        # Check whether the collision is on the left, else right
                        if overlap[0] == obj_rect[0]:
                            left = True

                        # Check whether the collision is on the top, else bottom
                        if overlap[1] == obj_rect[1]:
                            top = True

                        x_overlap = overlap[2]
                        y_overlap = overlap[3]

                        # If y_overlap is negative, the collision is on top
                        return(x_overlap, y_overlap, left, top, tile_rect, meta)

                index = index + 1

            index = index + self.offset
        
        return(None, None, None, None, None, None)

    def point_collision(self, (px, py)):
        """Returns the index of the tile that the point is on"""

        x = 0
        y = 0
        index = self.start
        for j in range(self.screen_height_tiles):
            y = (j * self.tile_height) - self.y_offset
            for i in range(self.screen_width_tiles):
                x = (i * self.tile_width) - self.x_offset
                tile = self.data[index]

                # If we have a non-blank tile
                if px < x:
                    pass
                elif py < y:
                    pass
                elif px > x + self.tile_width:
                    pass
                elif py > y + self.tile_height:
                    pass
                else:
                    return index

                index = index + 1

            index = index + self.offset

        return -1

    def translate_screen_to_map(self, (sx, sy)):
        """Translate x/y coordinates on the screen to x/y coordinates on the map"""

        rx = 0
        ry = 0

        rx = sx + self.viewport_x
        ry = sy + self.viewport_y

        return (rx, ry)

    def set_viewport(self, (viewport_x, viewport_y)):
        """Set the start tile, and x/y offsets given x,y position (on the map) as top left of the viewport"""

        current_x = self.viewport_x
        current_y = self.viewport_y

        # No need to update if no change
        if (viewport_x == 0) and (viewport_y == 0):
            return

        if (0 > viewport_x):
            viewport_x = 0
        if (0 > viewport_y):
            viewport_y = 0

        cols = int(viewport_x / self.tile_width)
        rows = int(viewport_y / self.tile_height)

        xoffset = viewport_x % self.tile_width
        yoffset = viewport_y % self.tile_height

        # Make sure column location is within map proper
        if ((cols + self.screen_width_tiles) > self.map_width_tiles):
            cols = int(self.map_width_tiles - self.screen_width_tiles)
            xoffset = self.tile_width

        # Make sure row location is within map proper
        if ((rows + self.screen_height_tiles) > self.map_height_tiles):
            rows = int(self.map_height_tiles - self.screen_height_tiles)
            yoffset = self.tile_height

        # Record location for map draw
        self.start = (rows * self.map_width_tiles) + cols
        self.x_offset = xoffset
        self.y_offset = yoffset

        self.viewport_x = (cols * self.tile_width) + xoffset
        self.viewport_y = (rows * self.tile_height) + yoffset

        return (current_x - self.viewport_x, current_y - self.viewport_y)

    def set_tile(self, index, value, meta):
        """Set a given tile"""

        self.data[index] = value
        self.meta[index] = meta


    def get_x(self):
        """Return the "viewport_x" position of the map"""
        return self.viewport_x

    def get_y(self):
        """Return the "viewport_y" position of the map"""
        return self.viewport_y

map = Map()
