"""
    The main character
"""

from map import map
from input import input
from config import config
from object import Object
from constants import *

class Player(Object):

    # This is called after Object's constructor, automatically
    def init(self):
        # Setup constants
        self.width = 16
        self.height = 16

        self.jump = float(7.5)
        self.jump_decay = 0.2
        self.gravity = 2.8
        self.falling = True
        self.default_x_vel = 2.9
        self.default_y_vel = 2.9

        self.load_tiles("mrbot.png")

    # Handle updates to movement, animation, etc...
    def update(self, time):

        self.handle_input()

        self.animation_update(time)
        self.jump_update()

        # Add in gravity
        if True == self.falling:
            self.add_velocity((0, self.gravity))

        if True == self.jumping:
            self.add_velocity((0, -self.jump_current))

        self.move((self.x_vel, self.y_vel), config.get_width(), config.get_height())

        # Now clear out old velocity
        self.set_velocity()

    def handle_input(self):
        """Handle input and collision detection"""

        # Move the player around
        if input.get_left():
            self.add_velocity((-self.default_x_vel, 0))
        elif input.get_right():
            self.add_velocity((self.default_x_vel, 0))

        # Set the jumping state
        if input.get_jump() and (None == self.falling) and (None == self.jumping):
            self.start_jumping()
        elif (None == input.get_jump()):
            self.stop_jumping()

        # The square over the player
        (t_x, t_y, t_left, t_top, t_rect, t_meta) = map.collision(self.type, self.x, self.y - 4, self.width, 4)
        if (None != t_y) and (True == self.jumping):
            self.set_velocity((self.x_vel, 0))
            self.set((None, t_rect[1] + t_rect[3]))
            self.stop_jumping()
            input.toggle_jump() # Force input to stop showing jump
            self.start_falling()

        # The the object's collision state
        (my_x, my_y, my_left, my_top, my_rect, my_meta) = map.object_collision(self)
        if (None != my_x):
            # Check for my_left or right collision
            if (True == my_left):
                self.set_velocity((0, self.y_vel))
                self.set((my_rect[0] + my_rect[2], None))
            else:
                self.set_velocity((0, self.y_vel))
                self.set((my_rect[0] - self.width, None))

        # The square under the player
        (b_x, b_y, b_left, b_top, b_rect, b_meta) = map.collision(self.type, self.x, self.y + self.height, self.width, 4)
        if (None != b_y) and (True == self.falling):
            self.set_velocity((self.x_vel, 0))
            self.set((None, b_rect[1] - self.height))
            self.stop_falling()
        elif (None == b_y):
            self.start_falling()

        # The square left of the player
        (l_x, l_y, l_left, l_top, l_rect, l_meta) = map.collision(self.type, self.x - 4, self.y, 4, self.height)
        if (None != l_y) and (self.x_vel < 0):
            self.set_velocity((0, self.y_vel))
            self.set((l_rect[0] + l_rect[2], None))

        # The square right of the player
        (r_x, r_y, r_left, r_top, r_rect, r_meta) = map.collision(self.type, self.x + self.width, self.y, 4, self.height)
        if (None != r_y) and (self.x_vel > 0):
            self.set_velocity((0, self.y_vel))
            self.set((r_rect[0] - self.width, None))


